package mx.transport.recharge.base;

import android.app.AlertDialog;
import android.content.Intent;
import android.nfc.NfcAdapter;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;


import com.google.android.material.snackbar.Snackbar;

import javax.inject.Inject;
import javax.inject.Named;

import mx.transport.recharge.R;
import mx.transport.recharge.activities.AuthActivity;
import mx.transport.recharge.activities.MainActivity;
import mx.transport.recharge.activities.SplashActivity;
import mx.transport.recharge.controller.MainController;
import mx.transport.recharge.custom.MProgressDialog;
import mx.transport.recharge.di.manager.DaggerManager;
import mx.transport.recharge.di.manager.NavigatorManager;
import mx.transport.recharge.listener.DialogListener;
import mx.transport.recharge.preference.RechargePreference;
import mx.transport.recharge.realm.RealmConfiguration;
import mx.transport.recharge.util.PermissionUtils_;

import static mx.transport.recharge.util.constants.RechargeConstants.ACTIVITY_AUTH;
import static mx.transport.recharge.util.constants.RechargeConstants.ACTIVITY_MAIN;
import static mx.transport.recharge.util.constants.RechargeConstants.ACTIVITY_SPLASH;

public class ActivityBase extends AppCompatActivity {

    private static final String TAG = ActivityBase.class.getName();
    public PermissionUtils_ permissionUtils;
    private MProgressDialog progressDialog = null;
    public NfcAdapter mAdapter;

    @Inject
    public NavigatorManager navigatorManager;

    @Inject
    public RechargePreference preferences;

    @Inject
    @Named("main")
    public MainController mainController;

    @Inject
    public RealmConfiguration realmConfiguration;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        DaggerManager.component().inject(this);
        super.onCreate(savedInstanceState);
    }

    public void onNavigate(int idView, Bundle bundle) {
        try {
            Intent intent = null;
            switch (idView) {
                case ACTIVITY_SPLASH:
                    intent = new Intent(this, SplashActivity.class);
                    break;
                case ACTIVITY_AUTH:
                    intent = new Intent(this, AuthActivity.class);
                    break;
                case ACTIVITY_MAIN:
                    intent = new Intent(this, MainActivity.class);
                    break;
                default:
                    Log.d(TAG, "navigation default");
                    break;
            }
            if (bundle != null && intent != null) {
                intent.putExtras(bundle);
            }
            if (intent != null) {
                startActivity(intent);
                this.finish();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized void showProgress() {
        try {
            this.runOnUiThread(() -> {
                if (progressDialog == null) {
                    progressDialog = (MProgressDialog) new MProgressDialog(this).ctor();
                    if (!progressDialog.isShowing()) {
                        progressDialog.show();
                    }
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized void hiddenProgress() {
        try {
            this.runOnUiThread(() -> {
                if (progressDialog != null) {
                    progressDialog.dismiss();
                    progressDialog = null;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        permissionUtils.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    public synchronized void showExitDialog(DialogListener dialogListener) {
        AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(this);
        alertDialogBuilder.setMessage(R.string.lbl_exit_message).setCancelable(false).setPositiveButton(android.R.string.yes, (dialog, id) -> dialogListener.onAccept()).setNegativeButton(android.R.string.cancel, null);
        AlertDialog alertDialog = alertDialogBuilder.create();
        alertDialog.show();
    }

    public void showMessageDialog(String message) {
        AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(this);
        alertDialogBuilder
                .setTitle(R.string.lbl_message_dialog)
                .setMessage(message)
                .setCancelable(false)
                .setPositiveButton(android.R.string.yes, null);
        AlertDialog alertDialog = alertDialogBuilder.create();
        alertDialog.show();
    }

    public synchronized void showMessage(String message) {
        if (!TextUtils.isEmpty(message)) {
            Snackbar snackbar = Snackbar.make(this.getWindow().getDecorView(), message, Snackbar.LENGTH_SHORT);
            snackbar.show();
        }
    }

}
