package mx.transport.recharge.di.manager;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBar;
import android.text.TextUtils;
import android.util.Log;
import android.widget.TextView;

import com.google.android.material.bottomnavigation.BottomNavigationView;

import mx.transport.recharge.R;
import mx.transport.recharge.ui.history.HistoryFragment;
import mx.transport.recharge.ui.configuration.ConfigurationFragment;
import mx.transport.recharge.ui.consulting.ConsultingFragment;
import mx.transport.recharge.ui.home.HomeFragment;
import mx.transport.recharge.ui.recharger.RechargerFragment;
import mx.transport.recharge.ui.buy.BuyFragment;

public class NavigatorManager implements FragmentManager.OnBackStackChangedListener {

    private final static String LOG_TAG = NavigatorManager.class.getName();
    private Context mContext;
    private String TAG;
    private FragmentManager mFragmentManager;
    private Fragment fragment;
    private NavigationListener mNavigationListener;
    private TextView mToolbarTitle;
    private ActionBar mActionBar;
    private BottomNavigationView bottomNavigationView;

    public static final int F_LOGIN = 1;


    public static final int F_MAIN = 10;
    public static final int F_CONFIGURATION = 11;

    public static final int F_RECHARGER     = 20;
    public static final int F_CONSULTING    = 30;
    public static final int F_BUY           = 40;

    public static final int F_ASSIGNMENT = 50;



    public interface NavigationListener {
        void onBackstackChange();
    }

    public void initNavigation(FragmentManager fragmentManager, TextView toolbarTitle, NavigationListener navigationListener, BottomNavigationView bottomNavigationView) {
        this.mToolbarTitle = toolbarTitle;
        this.mFragmentManager = fragmentManager;
        this.mNavigationListener = navigationListener;
        this.bottomNavigationView = bottomNavigationView;
        this.mFragmentManager.addOnBackStackChangedListener(() -> {
            if (mNavigationListener != null) {
                mNavigationListener.onBackstackChange();
            }
        });
    }

    public synchronized void navigation(int idNavigation, String tag, Bundle bundle) {
        switch (idNavigation) {
            case F_MAIN:
                fragment = HomeFragment.newInstance();
                break;
            case F_CONFIGURATION:
                fragment = ConfigurationFragment.newInstance();
                break;
            case F_RECHARGER:
                fragment = RechargerFragment.newInstance();
                break;
            case F_CONSULTING:
                fragment = ConsultingFragment.newInstance();
                break;
            case F_BUY:
                fragment = BuyFragment.newInstance();
                break;
            case F_ASSIGNMENT:
                fragment = HistoryFragment.newInstance();
                break;

            default:
                break;
        }
        if (fragment instanceof HomeFragment || fragment instanceof HistoryFragment) {
            openAsRoot(fragment, tag);
        } else {
            openFragment(fragment, tag);
        }
    }

    private void openAsRoot(Fragment fragment, String tag) {
        popAllFragments();
        openFragment(fragment, tag);
    }

    private void openFragment(Fragment fragment, String tag) {
        if (mFragmentManager != null) {
            mFragmentManager.beginTransaction().replace(R.id.fragment_container, fragment, tag).setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE).addToBackStack(fragment.toString()).commit();
            setToolbarTitle(tag);
        }
        int level = getNavStackEntryCount();
        Log.d(LOG_TAG, "level Fragments: " + level);
    }

    private void popAllFragments() {
        int backStackCount = getNavStackEntryCount();
        if (backStackCount >= 1) {
            Log.d(LOG_TAG, "popAllFragments: " + backStackCount);
            for (int f = 0; f < backStackCount; f++) {
                int backStackId = mFragmentManager.getBackStackEntryAt(f).getId();
                mFragmentManager.popBackStack(backStackId, FragmentManager.POP_BACK_STACK_INCLUSIVE);
            }
        }
    }

    public void navigateBack(Activity activity) {
        int navigateCount = getNavStackEntryCount();
        Log.d(LOG_TAG, "navigateBack: " + navigateCount);
        if (navigateCount == 0) {
            activity.finish();
        } else {
            mFragmentManager.popBackStackImmediate();
        }
    }

    public int getNavStackEntryCount() {
        return (mFragmentManager != null) ? mFragmentManager.getBackStackEntryCount() : 0;
    }

    public void restoreToolbar() {
        try {
            Fragment fragment = mFragmentManager.findFragmentById(R.id.fragment_container);
            if(fragment != null) {
                if (mToolbarTitle != null) {
                    mToolbarTitle.setText(fragment.getTag());
                }
                Log.d(TAG, "restoreToolbar::: " + fragment.getTag());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public boolean isRootFragmentVisible() {
        return mFragmentManager.getBackStackEntryCount() <= 1;
    }

    @Override
    public void onBackStackChanged() {
        if (mNavigationListener != null) {
            mNavigationListener.onBackstackChange();
        }
    }

    public void setToolbarTitle(String title) {
        if (!TextUtils.isEmpty(title) && mToolbarTitle != null) {
            mToolbarTitle.setText(title);
        }
    }

}
