package mx.transport.recharge.realm;

import java.util.List;

import io.realm.Realm;
import mx.transport.recharge.listener.RealmTransactionListener;
import mx.transport.recharge.model.Blacklist;
import mx.transport.recharge.model.CardTypes;
import mx.transport.recharge.model.Companies;
import mx.transport.recharge.model.CostLimit;
import mx.transport.recharge.model.Movements;
import mx.transport.recharge.model.PointOfSale;
import mx.transport.recharge.model.Product;
import mx.transport.recharge.model.RechargeLimit;
import mx.transport.recharge.model.UserInfo;

public class RealmConfiguration {

    private Realm realm;

    public RealmConfiguration(Realm realm) {
        this.realm = realm;
    }

    public List<CardTypes> getCardTypes(){
        return realm.copyFromRealm(realm.where(CardTypes.class).findAll());
    }

    public CardTypes getCardType(String type){
        return realm.copyFromRealm(realm.where(CardTypes.class).equalTo("cve", type).findFirst());
    }

    public Companies getCompanies(String type){
        return realm.copyFromRealm(realm.where(Companies.class).equalTo("cve", type).findFirst());
    }

    public void insertCardType(List<CardTypes> cardTypes, RealmTransactionListener listener){
           realm.executeTransactionAsync(realm -> {
                realm.copyToRealmOrUpdate(cardTypes);
            }, () -> {
                listener.onRealTransactionSuccess();
            }, error -> {
                listener.onRealmTransactionError(error);
            });
    }

    public void insertProduct(List<RechargeLimit> rechargeLimitList, List<CostLimit> costLimitList) {
        realm.executeTransactionAsync(realm -> {
            realm.copyToRealmOrUpdate(rechargeLimitList);
            realm.copyToRealmOrUpdate(costLimitList);
        }, () -> {

        }, error -> {

        });
    }

    public Product getProduct(){
        Product product = null;
        try {
            List<RechargeLimit> limits = realm.copyFromRealm(realm.where(RechargeLimit.class).findAll());
            List<CostLimit> costs = realm.copyFromRealm(realm.where(CostLimit.class).findAll());
            product = new Product(Double.parseDouble(costs.get(0).getCosto()), Double.parseDouble(limits.get(0).getTop()));
        } catch (Exception e){
            e.printStackTrace();
        }
        return product;
    }

    public void insertUser(List<PointOfSale> pointOfSaleList, List<Companies> companiesList) {
        realm.executeTransactionAsync(realm -> {
            realm.copyToRealmOrUpdate(pointOfSaleList);
            realm.copyToRealmOrUpdate(companiesList);
        }, () -> {

        }, error -> {

        });
    }

    public UserInfo getUser(){
        UserInfo userInfo = null;
        try {
            List<PointOfSale> pointOfSales = realm.copyFromRealm(realm.where(PointOfSale.class).findAll());
            List<Companies> companies = realm.copyFromRealm(realm.where(Companies.class).findAll());

            userInfo = new UserInfo(companies.get(0).getCve(),
                    companies.get(0).getDes(),
                    pointOfSales.get(0).getDes(),
                    pointOfSales.get(0).getTer(),
                    pointOfSales.get(0).getCve(),
                    pointOfSales.get(0).getMoneda());
        } catch (Exception e){
            e.printStackTrace();
        }
        return userInfo;
    }


    public void insertBlackList(List<Blacklist> blacklistList) {
        realm.executeTransactionAsync(realm -> {
            realm.copyToRealmOrUpdate(blacklistList);
        }, () -> {

        }, error -> {

        });
    }

    public List<Blacklist> getBlackList() {
        return realm.copyFromRealm(realm.where(Blacklist.class).findAll());
    }

    public void insertMovements(List<Movements> movementsList) {
        realm.executeTransactionAsync(realm -> {
            realm.copyToRealmOrUpdate(movementsList);
        }, () -> {

        }, error -> {

        });
    }

    public Movements getMovement(String c){
        Movements movements = null;

        movements =  realm.where(Movements.class).equalTo("cve_mov", c).findFirst();

        return movements;
    }

    public List<Movements> getMovements(){
        return realm.copyFromRealm(realm.where(Movements.class).findAll());
    }


    public void clear() {
        realm.executeTransaction(r -> r.deleteAll());
    }


    public void closeRealm() {
        realm.close();
    }

    //https://github.com/DroidsOnRoids/ExampleRealmMVP-Android/tree/master/app/src/main/java/pl/droidsonroids/examplerealmmvp

}
