package mx.transport.recharge.ui.buy;

import android.nfc.Tag;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;


import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import mx.transport.recharge.R;
import mx.transport.recharge.activities.MainActivity;
import mx.transport.recharge.base.FragmentBase;
import mx.transport.recharge.custom.DataPickerFragment;
import mx.transport.recharge.listener.AmountLister;
import mx.transport.recharge.model.Blacklist;
import mx.transport.recharge.model.CardBuy;
import mx.transport.recharge.model.CardTypes;
import mx.transport.recharge.model.Product;
import mx.transport.recharge.model.UserBuy;
import mx.transport.recharge.model.UserInfo;
import mx.transport.recharge.util.constants.RechargeConstants;


public class BuyFragment extends FragmentBase implements BuyView, AdapterView.OnItemSelectedListener {

    private static final String TAG = BuyFragment.class.getName();
    private BuyPresenter presenter;
    private List<Blacklist> blacklists = new ArrayList<>();
    private UserInfo info;
    private Product product;

    private Tag tag;

    boolean scan = false;

    private String type;
    private CardBuy card;
    private UserBuy user;


    private boolean customization;
    private String birthday;
    private String vigency;

    public static BuyFragment newInstance() {
        return new BuyFragment();
    }

    @BindView(R.id.tv_company)
    TextView companyTv;
    @BindView(R.id.tv_point_sale)
    TextView pointSaleTv;

    @BindView(R.id.ed_price)
    EditText priceEd;
    @BindView(R.id.ed_add)
    EditText addEd;
    @BindView(R.id.ed_total)
    EditText totalEd;

    @BindView(R.id.sp_card_type)
    Spinner cardTypeSp;

    @BindView(R.id.switchCustom)
    Switch informationSwitch;

    @BindView(R.id.tie_name)
    EditText edName;
    @BindView(R.id.tie_ap)
    EditText edAp;
    @BindView(R.id.tie_am)
    EditText edAm;
    @BindView(R.id.tie_email)
    EditText edEmail;
    @BindView(R.id.tie_phone)
    EditText edPhone;
    @BindView(R.id.tie_birthday)
    EditText edBirthDay;

    @BindView(R.id.linearInformation)
    View informationLinear;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rootView = inflater.inflate(R.layout.fragment_buy, container, false);
        unbinder = ButterKnife.bind(this, rootView);
        presenter = new BuyPresenter(activity, mainController, preferences, realmConfiguration, this);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        boolean isEnabled = ((MainActivity) activity).mAdapter.isEnabled();
        /*if (isEnabled) {
            ((MainActivity) activity).onSetupForegroundDispatch();
        }*/

        try {
            blacklists = realmConfiguration.getBlackList();
            info = realmConfiguration.getUser();
            companyTv.setText(getString(R.string.lbl_company) + ": " + info.getCompany());
            pointSaleTv.setText(getString(R.string.lbl_point_sale) + ": " + info.getPoint());
            product = realmConfiguration.getProduct();
            priceEd.setText("$" + product.getCosto());
            totalEd.setText("$" + product.getCosto());
            cardTypeSp.setAdapter(util.buildArrayAdapter(getContext(), realmConfiguration.getCardTypes()));
            cardTypeSp.setOnItemSelectedListener(this);
        } catch (Exception e) {
            e.printStackTrace();
        }

        informationSwitch.setOnCheckedChangeListener((compoundButton, b) -> {
            new Handler().post(() -> {
                customization = b;
                informationLinear.setVisibility(b ? View.VISIBLE : View.GONE);
            });
        });

        addEd.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                int size = charSequence.length();
                System.out.println("SIZE" + size);
                util.validateAmount(charSequence.toString(), product.getCosto(), product.getTope(), new AmountLister() {

                    @Override
                    public void onAmountUpdate(double amount) {
                        totalEd.setText("$" + amount);
                    }

                    @Override
                    public void onAmountError(String message) {
                        addEd.setText("");
                        totalEd.setText("$" + product.getCosto());
                        activity.showMessageDialog(message);
                    }

                    @Override
                    public void onAmountReset() {
                        totalEd.setText("$" + product.getCosto());
                    }
                });

            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
    }

    @Override
    public void onResume() {
        super.onResume();
        //((MainActivity) activity).onEnableForegroundDispatch();
    }

    @Override
    public void onPause() {
        super.onPause();
        //((MainActivity) activity).onDisableForegroundDispatch();
    }

    /*@Override
    public void onCardRead(Tag tag) {
        this.tag = tag;
        new ReadCardTask(activity, tag, this).execute();
    }*/

    @OnClick({R.id.tie_birthday, R.id.button_write})
    public synchronized void onActions(View view) {
        switch (view.getId()) {
            case R.id.tie_birthday:
                setBirthDate();
                break;
            case R.id.button_write:
                if (customization) {
                    onCardProcess();
                } else {
                    if(util.quantity(addEd.getText().toString()) >= 1){
                        activity.showProgress();
                    } else {
                        activity.showMessageDialog(getString(R.string.lbl_error_quantity));
                        addEd.requestFocus();
                    }
                }
                break;
        }
    }

    public synchronized void setBirthDate() {
        try {
            DataPickerFragment pickerFragment = DataPickerFragment.newInstance((datePicker, year, month, day) -> {
                String mDay = day < 9 ? "0" + day : "" + day;
                String mMont = (month + 1) < 9 ? "0" + (month + 1) : "" + (month + 1);
                final String selectDate = mDay + "/" + mMont + "/" + year;
                birthday = year + "-" + mMont + "-" + mDay;
                edBirthDay.setText(selectDate);
                util.hideKeyboard();
            });
            pickerFragment.show(activity.getFragmentManager(), "datePicker");
        } catch (Exception e) {
            Toast.makeText(activity, activity.getString(R.string.lbl_error_calendar), Toast.LENGTH_LONG).show();
        }
    }

    public boolean onCardProcess() {
        boolean success = true;
        StringBuilder builder = new StringBuilder();
        String name = edName.getText().toString();
        String ap = edAp.getText().toString();
        String am = edAm.getText().toString();
        String surename = ap + " " + am;
        String email = edEmail.getText().toString();
        String phone = edPhone.getText().toString();
        String birthdate = edBirthDay.getText().toString();

        user = new UserBuy(name, surename, birthdate, email, phone);

        if(util.quantity(addEd.getText().toString()) == 0){
            builder.append(getString(R.string.lbl_error_quantity));
            success = false;
        }

        if (TextUtils.isEmpty(name)) {
            builder.append(getString(R.string.lbl_error_name));
            success = false;
        }
        if (TextUtils.isEmpty(ap)) {
            builder.append(getString(R.string.lbl_error_ap));
            success = false;
        }
        if (TextUtils.isEmpty(am)) {
            builder.append(getString(R.string.lbl_error_am));
            success = false;
        }
        if (TextUtils.isEmpty(email)) {
            builder.append(getString(R.string.lbl_error_email));
            success = false;
        } else {
            if(!util.isEmailValid(email)){
                builder.append(getString(R.string.lbl_error_email_valid));
                success = false;
            }
        }

        if (TextUtils.isEmpty(birthdate)) {
            builder.append(getString(R.string.lbl_error_birthdate));
            success = false;
        }


        if (success) {
            //presenter.loginService(user, pass);
            activity.showProgress();
        } else {
            activity.showMessageDialog(builder.toString());
        }

        return success;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (unbinder != null) {
            unbinder.unbind();
        }
    }

    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        try {
            CardTypes classType = (CardTypes) parent.getAdapter().getItem(position);
            vigency = (classType.getCve().equals(RechargeConstants.TYPE_STUDENT) ? vigency = util.getDateYear() : "");
            Log.d(TAG, "V:" + vigency);
            type = classType.getCve();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {

    }

    /*@Override
    public void onCardReadComplete(CardProperties card) {
        activity.hiddenProgress();
        try {
            String price = priceEd.getText().toString();
            String netCharge = addEd.getText().toString();
            presenter.onCardRegister(tag, card, price, netCharge, type, customization, user);
        } catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void onCardReadInvalidCard() {
        activity.hiddenProgress();
        activity.showMessageDialog(getString(R.string.lbl_message_card_invalid));
    }

    @Override
    public void onCardReadNotFound() {
        activity.hiddenProgress();
        activity.showMessageDialog(getString(R.string.lbl_message_card_not_fount));
    }

    @Override
    public void onCardReadExist() {
        activity.showMessage(getString(R.string.lbl_message_card_exist));
    }

    @Override
    public void onCardReadException(Throwable throwable) {
        throwable.printStackTrace();
    }*/

    @Override
    public void onSuccessBuy() {
        try {
            addEd.setText("");
            edName.setText("");
            edAp.setText("");
            edAm.setText("");
            edEmail.setText("");
            edPhone.setText("");
            edBirthDay.setText("");
            informationSwitch.setChecked(false);
            onBackNavigation();
        } catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void onSuccessWrite() {
        onBackNavigation();
    }

}
