package mx.transport.recharge.ui.home;

import android.util.Log;

import com.google.gson.Gson;

import mx.transport.recharge.R;
import mx.transport.recharge.base.ActivityBase;
import mx.transport.recharge.base.BaseResponse;
import mx.transport.recharge.base.FragmentBase;
import mx.transport.recharge.controller.MainController;
import mx.transport.recharge.listener.HandleListener;
import mx.transport.recharge.listener.RealmTransactionListener;
import mx.transport.recharge.model.Configuration;
import mx.transport.recharge.model.response.ConfigurationResponse;
import mx.transport.recharge.preference.RechargePreference;
import mx.transport.recharge.realm.RealmConfiguration;
import mx.transport.recharge.util.RechargeUtil;
import mx.transport.recharge.util.constants.RechargeConstants;

public class HomePresenter implements HandleListener, BaseResponse, RealmTransactionListener {

    private static final String TAG = HomePresenter.class.getSimpleName();
    private ActivityBase activity;
    private FragmentBase fragmentBase;
    private MainController controller;
    private RechargePreference preferences;
    private HomeView view;
    private RechargeUtil util;
    private String imei;
    private RealmConfiguration configurationService;

    public HomePresenter(FragmentBase fragmentBase, MainController controller, RechargePreference preferences, RealmConfiguration configurationService, HomeView view) {
        this.activity = fragmentBase.activity;
        this.fragmentBase = fragmentBase;
        this.controller = controller;
        this.view = view;
        this.util = RechargeUtil.getInstance(activity);
        this.preferences = preferences;
        this.imei = preferences.getStringData(RechargeConstants.PF_IMEI);
        this.configurationService = configurationService;
        Log.d(TAG, "CARD TYPES: " + configurationService.getCardTypes().size());
    }

    public void configurationService(){
        boolean callApi = util.validateDate(preferences);
        System.out.println("CALLAPI: " + callApi);
        if(callApi){
            if (util.isConnected()) {
                String date = util.getDate();
                Configuration configuration = new Configuration(imei, date);
                controller.callConfiguration(configuration).subscribe(this::onHandleSuccess, this::onHandleFailed);
            } else {
                activity.showMessage(activity.getString(R.string.lbl_internet_connection));
            }
        }
    }

    @Override
    public void onHandleSuccess(Object object) {
        try {
            mx.transport.recharge.model.response.BaseResponse baseResponse = (mx.transport.recharge.model.response.BaseResponse) object;
            if (baseResponse != null) {
                util.onResponseValidate(this, object, 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        activity.hiddenProgress();
    }

    @Override
    public void onHandleFailed(Throwable t) {
       activity.showMessage(t.getMessage());
       activity.hiddenProgress();
    }

    @Override
    public void onSuccess(String json) {
        try {
            ConfigurationResponse response = new Gson().fromJson(json, ConfigurationResponse.class);
            if (response != null) {
                preferences.setStringData(RechargeConstants.PF_LAST_UPDATE, util.getDate());
                configurationService.insertCardType(response.getCardTypesList(), this);
                configurationService.insertProduct(response.getRechargeLimitList(), response.getCostLimitList());
                configurationService.insertUser(response.getPointOfSaleList(), response.getCompaniesList());
                configurationService.insertBlackList(response.getBlacklistList());
                configurationService.insertMovements(response.getMovementsList());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onSuccessRequest(int idRequest, Object obj) {

    }

    @Override
    public void onError(String message) {
       activity.showMessage(message);
    }

    @Override
    public void onRealTransactionSuccess() {

    }

    @Override
    public void onRealmTransactionError(Throwable error) {

    }


    // https://medium.com/@ssaurel/create-a-nfc-reader-application-for-android-74cf24f38a6f
   // https://www.codexpedia.com/android/android-nfc-read-and-write-example/
    // https://code.tutsplus.com/es/tutorials/reading-nfc-tags-with-android--mobile-17278 TODO API
    // https://developer.android.com/guide/topics/connectivity/nfc/nfc
    // https://www.learn2crack.com/2016/10/android-reading-and-writing-nfc-tags.html
    // https://bluefletch.com/blog/near-field-communication-reading-tags-with-android/
    // https://www.developer.com/ws/android/nfc-programming-in-android.html

}
