package mx.transport.recharge.ui.login;

import android.text.TextUtils;

import com.google.gson.Gson;

import mx.transport.recharge.R;
import mx.transport.recharge.base.ActivityBase;
import mx.transport.recharge.base.BaseResponse;
import mx.transport.recharge.controller.MainController;
import mx.transport.recharge.listener.HandleListener;
import mx.transport.recharge.model.response.UserResponse;
import mx.transport.recharge.model.Login;
import mx.transport.recharge.preference.RechargePreference;
import mx.transport.recharge.util.PermissionUtils_;
import mx.transport.recharge.util.RechargeUtil;
import mx.transport.recharge.util.constants.RechargeConstants;
import static android.Manifest.permission.READ_PHONE_STATE;
import static mx.transport.recharge.util.constants.RechargeConstants.ID_PHONE_PERMISSION;


public class LoginPresenter implements HandleListener, BaseResponse {

    private ActivityBase activity;
    private MainController controller;
    private RechargePreference preferences;
    private LoginView view;
    private RechargeUtil util;
    private PermissionUtils_ permissionUtils;

    public LoginPresenter(ActivityBase activity, MainController controller, RechargePreference preferences, PermissionUtils_ permissionUtils, LoginView view) {
        this.activity = activity;
        this.controller = controller;
        this.view = view;
        this.util = RechargeUtil.getInstance(activity);
        this.preferences = preferences;
        this.permissionUtils = permissionUtils;
    }

    public void validatePermissions(PermissionUtils_.RequestPermissionListener permissionListener) {
        permissionUtils.requestPermission(new String[]{READ_PHONE_STATE}, ID_PHONE_PERMISSION, permissionListener);
    }

    public void loginService(String user, String password) {
        if (util.isConnected()) {
            String imei = util.getDeviceIMEI();
            //String imei = "356021081495810";
            if(TextUtils.isEmpty(imei)) {
                view.onLoginError(activity.getString(R.string.lbl_error_imei));
            } else {
                Login login = new Login(user, password, imei);
                preferences.setStringData(RechargeConstants.PF_IMEI, imei);
                controller.callLogin(login).subscribe(this::onHandleSuccess, this::onHandleSuccess);
                activity.showProgress();   
            }
        } else {
            view.onLoginError(activity.getString(R.string.lbl_internet_connection));
        }
    }

    @Override
    public void onHandleSuccess(Object object) {
        try {
            mx.transport.recharge.model.response.BaseResponse baseResponse = (mx.transport.recharge.model.response.BaseResponse) object;
            if (baseResponse != null) {
                util.onResponseValidate(this, object, 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        activity.hiddenProgress();
    }

    @Override
    public void onHandleFailed(Throwable t) {
        activity.showMessage(t.getMessage());
        activity.hiddenProgress();
    }

    @Override
    public void onSuccess(String json) {
        try {
            //TODO VALIDAR USUARIO
            UserResponse recharger = new Gson().fromJson(json, UserResponse.class);
            if (recharger != null) {
                preferences.saveUser(recharger.getUser());
                view.onLoginSuccess();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onSuccessRequest(int idRequest, Object obj) {

    }

    @Override
    public void onError(String message) {
        activity.showMessage(message);
    }


}
