package mx.transport.recharge.activities;


import android.nfc.NfcAdapter;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import mx.transport.recharge.BuildConfig;
import mx.transport.recharge.R;
import mx.transport.recharge.base.ActivityBase;
import mx.transport.recharge.ui.login.LoginPresenter;
import mx.transport.recharge.ui.login.LoginView;
import mx.transport.recharge.util.PermissionUtils;

import static mx.transport.recharge.util.constants.RechargeConstants.ACTIVITY_MAIN;

public class AuthActivity extends ActivityBase implements LoginView, PermissionUtils.RequestPermissionListener {

    private static final String TAG = AuthActivity.class.getName();
    private LoginPresenter presenter;
    private boolean permissions = false;

    @BindView(R.id.edit_user)
    EditText userEd;
    @BindView(R.id.edit_password)
    EditText passwordEd;
    @BindView(R.id.button_login)
    Button loginBtn;
    @BindView(R.id.tv_message)
    TextView messageTv;
    @BindView(R.id.tv_version)
    TextView versionTv;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_auth);
        ButterKnife.bind(this);
        permissionUtils = PermissionUtils.getInstance(this);
        presenter = new LoginPresenter(this, mainController, preferences, permissionUtils, this);
        presenter.validatePermissions(this);
        versionTv.setText(getString(R.string.lbl_version) + " " + BuildConfig.VERSION_NAME);
    }

    @OnClick(R.id.button_login)
    public void onAction(View view) {
        if (permissions) {
            validateInformation();
        } else {
            showMessage(getString(R.string.lbl_error_permissions));
            presenter.validatePermissions(this);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    public boolean validateInformation() {
        boolean data = true;
        StringBuilder builder = new StringBuilder();
        String user = userEd.getText().toString();
        String pass = passwordEd.getText().toString();
        if (TextUtils.isEmpty(user)) {
            builder.append(getString(R.string.lbl_error_user) + "\n");
            data = false;
        }
        if (TextUtils.isEmpty(pass)) {
            builder.append(getString(R.string.lbl_error_password));
            data = false;
        }
        if (data) {
            presenter.loginService(user, pass);
        } else {
            showMessageDialog(builder.toString());
        }
        return data;
    }

    @Override
    public void onLoginSuccess() {
        onNavigate(ACTIVITY_MAIN, null);
    }

    @Override
    public void onLoginError(String message) {
        showMessage(message);
    }

    @Override
    public void onSuccess() {
        permissions = true;
    }

    @Override
    public void onFailed() {
        permissions = false;
        showMessage(getString(R.string.lbl_message_permissions));
        presenter.validatePermissions(this);
    }

    /*
    //USUARIOD DE TIPO RECARGA
         msalome // RECARGA
    	"password":"d3mo43c","usuario":"demorec"

     */

}
