package mx.transport.recharge.activities;

import android.app.PendingIntent;
import android.content.Intent;
import android.content.IntentFilter;
import android.nfc.NfcAdapter;
import android.os.Bundle;
import android.os.Handler;
import androidx.annotation.NonNull;
import androidx.appcompat.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.google.android.material.bottomnavigation.BottomNavigationView;

import butterknife.BindView;
import butterknife.ButterKnife;
import mx.transport.recharge.R;
import mx.transport.recharge.base.ActivityBase;
import mx.transport.recharge.di.manager.NavigatorManager;
import mx.transport.recharge.listener.DialogListener;

import static mx.transport.recharge.util.constants.RechargeConstants.ACTIVITY_AUTH;
import static mx.transport.recharge.di.manager.NavigatorManager.F_ASSIGNMENT;
import static mx.transport.recharge.di.manager.NavigatorManager.F_CONFIGURATION;
import static mx.transport.recharge.di.manager.NavigatorManager.F_MAIN;

public class MainActivity extends ActivityBase implements NavigatorManager.NavigationListener, BottomNavigationView.OnNavigationItemSelectedListener {

    private static final String TAG = MainActivity.class.getSimpleName();
    private MenuItem settingMenu, exitMenu;
    private PendingIntent mPendingIntent;
    private IntentFilter [] mIntentFilters;
    private String [][] mTeachList;

    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.toolbar_title)
    TextView toolbarTitle;
    @BindView(R.id.nav_view)
    public BottomNavigationView bottomNavigationView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);
        setSupportActionBar(toolbar);
        bottomNavigationView.setOnNavigationItemSelectedListener(this);
        initialFragment();
        mAdapter = NfcAdapter.getDefaultAdapter(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        navigatorManager.initNavigation(getSupportFragmentManager(), toolbarTitle, this::onBackstackChange, bottomNavigationView);
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        //onHandleIntent(intent);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main_menu, menu);
        settingMenu = menu.findItem(R.id.action_settings);
        exitMenu = menu.findItem(R.id.action_close);
        return true;
    }

    void initialFragment() {
        navigatorManager.initNavigation(getSupportFragmentManager(), toolbarTitle, this::onBackstackChange, bottomNavigationView);
        navigatorManager.navigation(F_MAIN, getString(R.string.app_name), null);
    }

    @Override
    public void onBackstackChange() {
        navigatorManager.restoreToolbar();
        int size = getSupportFragmentManager().getBackStackEntryCount();
        if (size > 1) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
            getSupportActionBar().setDisplayShowHomeEnabled(true);
            bottomNavigationView.setVisibility(View.GONE);
            showMenuOptions(false);
        }

        if (navigatorManager.isRootFragmentVisible()) {
            bottomNavigationView.setVisibility(View.VISIBLE);
            invalidateOptionsMenu();
            showMenuOptions(true);
        }
    }

    private void showMenuOptions(boolean v){
        try{
            if(settingMenu != null)
                settingMenu.setVisible(v);
            if(exitMenu != null)
                exitMenu.setVisible(v);
        } catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void onBackPressed() {
        if (getSupportFragmentManager().getBackStackEntryCount() > 1) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(false);
            navigatorManager.navigateBack(this);
        }
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_settings:
                navigatorManager.navigation(F_CONFIGURATION, getString(R.string.lbl_settings), null);
                break;
            case R.id.action_close:
                showExitDialog(new DialogListener() {
                    @Override
                    public void onAccept() {
                        new Handler().post(() -> {
                            preferences.cleanPreferences();
                            realmConfiguration.clear();
                            onNavigate(ACTIVITY_AUTH, null);
                        });
                    }

                    @Override
                    public void onCancel() {

                    }
                });
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem menuItem) {
        try {
            switch (menuItem.getItemId()) {
                case R.id.navigation_home:
                    navigatorManager.navigation(F_MAIN, getString(R.string.title_home), null);
                    break;
                case R.id.navigation_assignment:
                    navigatorManager.navigation(F_ASSIGNMENT, getString(R.string.title_assignment), null);
                    break;

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }


   /* @Override
    public void onSetupForegroundDispatch(){
        try {
            IntentFilter filter = new IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED);
           // filter.addDataType("*");
            mIntentFilters = new IntentFilter[]{filter};
            mTeachList = new String[][]{ new String[]{MifareClassic.class.getName()}};
            mPendingIntent = PendingIntent.getActivity(
                    this,
                    0,
                    new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP),
                    0);
            onHandleIntent(getIntent());
        } catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void onHandleIntent(Intent intent){
        try {
            final String action = intent.getAction();
            final Tag tag = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
            if (NfcAdapter.ACTION_NDEF_DISCOVERED.equals(action) || NfcAdapter.ACTION_TECH_DISCOVERED.equals(action)) {
               // showMessage("Read TAG");
                Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.fragment_container);
                if(fragment != null){
                    if(fragment instanceof ConsultingFragment){
                        ((ConsultingFragment) fragment).onCardRead(tag);
                    } else if(fragment instanceof BuyFragment){
                        ((BuyFragment) fragment).onCardRead(tag);
                    } else if(fragment instanceof RechargerFragment){
                    ((RechargerFragment) fragment).onCardRead(tag);
                }
                }
            }
        } catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void onEnableForegroundDispatch(){
        if(mAdapter != null){
            mAdapter.enableForegroundDispatch(this, mPendingIntent, mIntentFilters, mTeachList);
        }
    }

    @Override
    public void onDisableForegroundDispatch(){
        if(mAdapter != null){
            mAdapter.disableForegroundDispatch(this);
        }
    }*/


}
