package mx.transport.recharge.preference;

import android.content.Context;
import android.content.SharedPreferences;

import io.reactivex.schedulers.Schedulers;
import mx.transport.recharge.util.constants.RechargeConstants;
import mx.transport.recharge.model.User;

public class RechargePreference {

    private SharedPreferences mSharedPreferences;

    public RechargePreference(Context context) {
        this.mSharedPreferences = context.getSharedPreferences("R3c4rg3", Context.MODE_PRIVATE);
    }

    public RechargePreference(SharedPreferences mSharedPreferences) {
        this.mSharedPreferences = mSharedPreferences;
    }

    public void setStringData(String key, String value) {
        mSharedPreferences.edit().putString(key, value).commit();
    }

    public String getStringData(String key) {
        return mSharedPreferences.getString(key, "");
    }

    public void setIntData(String key, int value) {
        mSharedPreferences.edit().putInt(key, value).commit();
    }

    public int getIntData(String key) {
        return mSharedPreferences.getInt(key, 0);
    }


    public void setBooleanData(String key, boolean value) {
        mSharedPreferences.edit().putBoolean(key, value).commit();
    }

    public boolean getBooleanData(String key) {
        return mSharedPreferences.getBoolean(key, false);
    }


    public synchronized void cleanPreferences() {
        mSharedPreferences.edit().clear().commit();
    }


    public void saveUser(User user) {
        Schedulers.io().createWorker().schedule(() -> {
            setStringData(RechargeConstants.PF_ID, user.getIdUser());
            setStringData(RechargeConstants.PF_NAME, user.getName());
            setStringData(RechargeConstants.PF_PROFILE, user.getProfile());
            setBooleanData(RechargeConstants.PF_SESSION, true);
        });
    }

    public User getUser(){
        return new User(getStringData(RechargeConstants.PF_ID), getStringData(RechargeConstants.PF_NAME), getStringData(RechargeConstants.PF_PROFILE));
    }

}
