package mx.transport.recharge.ui.configuration;


import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import mx.transport.recharge.R;
import mx.transport.recharge.base.FragmentBase;
import mx.transport.recharge.model.Movements;
import mx.transport.recharge.model.User;
import mx.transport.recharge.model.UserInfo;
import mx.transport.recharge.util.constants.RechargeConstants;


public class ConfigurationFragment extends FragmentBase implements ConfigurationView {

    private static final String TAG = ConfigurationFragment.class.getName();
    private ConfigurationPresenter presenter;
    private UserInfo info;

    @BindView(R.id.tie_name)
    EditText nameTie;
    @BindView(R.id.tie_profile)
    EditText profileTie;

    @BindView(R.id.tie_company)
    EditText companyTie;
    @BindView(R.id.tie_point_sale)
    EditText pointSaleTie;
    @BindView(R.id.tie_currency)
    EditText currencyTie;
    @BindView(R.id.tv_movements)
    TextView movementsTv;
    @BindView(R.id.tie_update_date)
    EditText updateDateTie;


    public static ConfigurationFragment newInstance() {
        return new ConfigurationFragment();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rootView = inflater.inflate(R.layout.fragment_configuration, container, false);
        unbinder = ButterKnife.bind(this, rootView);
        presenter = new ConfigurationPresenter(activity, mainController, preferences, realmConfiguration, this);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        loadInformation();
    }

    private void loadInformation() {
        try {
            User user = preferences.getUser();
            info = realmConfiguration.getUser();
            nameTie.setText(user.getName());
            profileTie.setText(user.getProfile());
            companyTie.setText(info.getCompany());
            pointSaleTie.setText(info.getPoint());
            currencyTie.setText(info.getCurrency());
            movementsTv.setText(buildMovements(realmConfiguration.getMovements()));
            updateDateTie.setText(preferences.getStringData(RechargeConstants.PF_LAST_UPDATE));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public String buildMovements(List<Movements> movements) {
        StringBuilder builder = new StringBuilder();
        for (Movements m : movements) {
            builder.append("-" + m.getDes() + "\n");
        }
        return builder.toString();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (unbinder != null) {
            unbinder.unbind();
        }
    }

}
