package mx.transport.recharge.util;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.TextView;

import com.telpo.tps550.api.util.StringUtil;
import com.telpo.tps550.api.util.SystemUtil;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import mx.transport.recharge.R;
import mx.transport.recharge.base.BaseResponse;
import mx.transport.recharge.listener.AmountLister;
import mx.transport.recharge.preference.RechargePreference;
import mx.transport.recharge.util.constants.RechargeConstants;

public class RechargeUtil {

    private static final String TAG = RechargeUtil.class.getSimpleName();
    private static RechargeUtil _instance = null;
    private static Activity activity;


    public RechargeUtil(Activity activity) {
        RechargeUtil.activity = activity;
    }

    public static synchronized RechargeUtil getInstance(Activity activity) {
        if (_instance == null) {
            _instance = new RechargeUtil(activity);
        }
        return _instance;
    }

    public static boolean isConnected() {
        ConnectivityManager cm = (ConnectivityManager) activity.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting();
    }

    public static void hideKeyboard() {
        InputMethodManager imm = (InputMethodManager) activity.getSystemService(Activity.INPUT_METHOD_SERVICE);
        View view = activity.getCurrentFocus();
        if (view == null) {
            view = new View(activity);
        }
        imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
    }

    @SuppressLint("MissingPermission")
    public String getDeviceIMEI() {
        String deviceUniqueIdentifier = null;
        TelephonyManager tm = (TelephonyManager) activity.getSystemService(Context.TELEPHONY_SERVICE);
        if (null != tm) {
            deviceUniqueIdentifier = tm.getDeviceId();
        }
        if (null == deviceUniqueIdentifier || 0 == deviceUniqueIdentifier.length()) {
            deviceUniqueIdentifier = Settings.Secure.getString(activity.getContentResolver(), Settings.Secure.ANDROID_ID);
        }
        return deviceUniqueIdentifier;
    }


    public String getSimpleDate() {
        String mDate = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy"); // hh:mm:ss a
            mDate = dateFormat.format(new Date());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return mDate;
    }

    public String getDate() {
        String mDate = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss a"); // hh:mm:ss a
            mDate = dateFormat.format(new Date());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return mDate;
    }

    public String getDate(Date date) {
        String mDate = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy"); // hh:mm:ss a
            mDate = dateFormat.format(date);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return mDate;
    }

    public String getDateYear() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.YEAR, 1);
        return getDate(calendar.getTime());
    }

    public boolean validateDate(RechargePreference preferences) {
        try {
            String pDate = preferences.getStringData(RechargeConstants.PF_LAST_UPDATE);
            if (TextUtils.isEmpty(pDate)) {
                System.out.println("pDate:" + pDate);
                return true;
            } else {

                SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");
                Date date = dateFormat.parse(pDate);
                Date sDate = new Date();

                System.out.println("pDate:" + pDate + " sDate");

                if (date.before(sDate)) {
                    // statement
                    return false;
                }

                if (date.after(sDate)) {
                    // statement
                    return false;
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            return true;
        }
        return true;
    }

    public static void onResponseValidate(BaseResponse view, Object object, int idRequest) {
        try {
            mx.transport.recharge.model.response.BaseResponse baseResponse = (mx.transport.recharge.model.response.BaseResponse) object;
            if (baseResponse != null) {
                if (!baseResponse.status) { //!= Constants.STATUS_SUCCESS
                    String message = baseResponse.getMessage();
                    if (message.contains("Unprocessable")) {
                        //    String error = baseResponse.getErrors().replace("{","").replace("}","").replace("[","").replace("]","");
                        //    view.onError(error);
                    } else {
                        view.onError(baseResponse.getMessage() + baseResponse.getResponse());
                    }
                } else {

                    if (idRequest > 0) {
                        if (idRequest == 99) {
                            view.onSuccessRequest(idRequest, baseResponse);
                        } else {
                            view.onSuccessRequest(idRequest, baseResponse.getResponse());
                        }
                    } else {
                        view.onSuccess(baseResponse.response);
                    }

                }
            } else {
                view.onError("Error no sé obtuvo respuesta");
            }
        } catch (Exception e) {
            e.printStackTrace();
            view.onError("Error al procesar la respuesta");
        }
    }


    public static ArrayAdapter<String> buildArrayAdapter(Context context, List<?> objects) {
        ArrayAdapter adapter = new ArrayAdapter(context, android.R.layout.simple_spinner_dropdown_item, objects) {

            @Override
            public boolean isEnabled(int position) {
                //return position != 0;
                return true;
            }

            @Override
            public View getDropDownView(int position, View convertView, ViewGroup parent) {
                View v = super.getDropDownView(position, convertView, parent);
                TextView tv = (TextView) v;
               /*if (position == 0) { //== 0
                    tv.setTextColor(Color.GRAY);
                } else {
                    tv.setTextColor(Color.BLACK);
                }*/
                tv.setTextColor(Color.GRAY);
                return v;
            }

        };
        return adapter;
    }

    public synchronized void validateAmount(String q, double p, double tope, AmountLister lister) {
        if (!TextUtils.isEmpty(q)) {
            double quantity = Double.parseDouble(q);
            double total = quantity + p;
            if (total <= tope) {
                lister.onAmountUpdate(total);
            } else {
                lister.onAmountError(activity.getString(R.string.lbl_amount_max) + " $" + tope);
            }
        } else {
            lister.onAmountReset();
        }

    }

    public int quantity(String quantity) {
        int q = 0;
        try{
            if(!TextUtils.isEmpty(quantity)){
                q = Integer.parseInt(quantity);
            }
        } catch (Exception e){
            e.printStackTrace();
        }
        return q;
    }

    public static boolean isEmailValid(String email) {
        String expression = "^[\\w\\.-]+@([\\w\\-]+\\.)+[A-Z]{2,4}$";
        Pattern pattern = Pattern.compile(expression, Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(email);
        return matcher.matches();
    }


    private boolean isTablet(){
        return activity.getResources().getBoolean(R.bool.isTablet);
    }

    public boolean isTelpo(){
        if(SystemUtil.getDeviceType() == StringUtil.DeviceModelEnum.TPS900.ordinal() || Build.MODEL.equals(RechargeConstants.TPS530)){
            return true;
        }
        return false;
    }

    public void getRequestOrientation(){
        if(isTelpo() || isTablet()){
            activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        } else {
            activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        }
    }


}
