package mx.cardnfc.task;

import android.app.Activity;
import android.app.ProgressDialog;
import android.nfc.Tag;
import android.nfc.tech.MifareClassic;
import android.os.AsyncTask;

import mx.cardnfc.model.Card;
import mx.cardnfc.model.CardProperties;
import mx.cardnfc.recharge.callback.WriteCardCallback;
import mx.cardnfc.util.CardUtil;

public class WriteCardTask extends AsyncTask<Void, Void, Void> {

    private final Tag tag;
    private final CardUtil cardUtil;
    private final WriteCardCallback callback;
    private final Card card;
    private CardProperties cardProperties;
    private ProgressDialog dialog;
    boolean success = false;
    private Throwable throwable;

    public WriteCardTask(Activity activity, Tag tag, Card card, WriteCardCallback callback) {
        this.tag = tag;
        this.card = card;
        this.callback = callback;
        this.cardUtil = CardUtil.getInstance();
        this.dialog = new ProgressDialog(activity);
    }

    @Override
    protected void onPreExecute() {
        super.onPreExecute();
        dialog.setMessage("Grabando información");
        dialog.show();
    }

    @Override
    protected Void doInBackground(Void... voids) {
        final MifareClassic mfc = MifareClassic.get(tag);
        try {
            Thread.sleep(5000);
            if (mfc != null) {
                mfc.connect();
                int t = mfc.getType();
                int s = mfc.getSize();
                boolean isValid = cardUtil.cardValidate(t, s);
                if (isValid) {
                    cardProperties = cardUtil.cardInformation(mfc);
                    if (card.getParam_2().equalsIgnoreCase(cardProperties.getCardId())) {
                        cardUtil.writeCard(mfc, card);
                        success = true;
                    } else {
                        callback.onCardWriteInvalidCard();

                    }
                } else {
                    callback.onCardWriteInvalidCard();
                }
            }
        } catch (Exception e) {
            callback.onCardWriteException(e);
            this.throwable = e;
        } finally {
            try {
                mfc.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Override
    protected void onPostExecute(Void aVoid) {
        super.onPostExecute(aVoid);
        if (dialog.isShowing()) {
            dialog.dismiss();
        }
        if(success){
            callback.onCardWrite();
        } else {
            callback.onCardWriteException(throwable);
        }
    }

}
