package mx.cardnfc.util;

import android.nfc.tech.MifareClassic;
import android.util.Log;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;

import mx.cardnfc.constants.CardConstants;
import mx.cardnfc.model.Card;
import mx.cardnfc.model.CardInformation;
import mx.cardnfc.model.CardProperties;

public class CardUtil {

    private static final String TAG = CardUtil.class.getSimpleName();
    private static CardUtil instance = null;
    private String CHARSET = "ISO-8859-1";


    public static CardUtil getInstance() {
        if (instance == null) {
            instance = new CardUtil();
        }
        return instance;
    }

    public CardProperties cardInformation(MifareClassic mfc) {
        CardProperties card = null;
        try {
            card = new CardProperties(convertBytesToString(mfc.getTag().getId()), mfc.getTag().getId(), cardType(mfc.getType()), mfc.getSectorCount(), mfc.getBlockCount(), mfc.getSize(), cardSize(mfc.getSize()));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return card;
    }

    public String cardType(int type) {
        switch (type) {
            case MifareClassic.TYPE_CLASSIC:
                return CardConstants.CARD_TYPE_CLASSIC;
            case MifareClassic.TYPE_PLUS:
                return CardConstants.CARD_TYPE_PLUS;
            case MifareClassic.TYPE_PRO:
                return CardConstants.CARD_TYPE_PRO;
            default:
                return CardConstants.CARD_TYPE_UNKNOWN;
        }
    }

    public String cardSize(int size) {
        switch (size) {
            case MifareClassic.SIZE_1K:
                return CardConstants.CARD_SIZE_1K;
            case MifareClassic.SIZE_2K:
                return CardConstants.CARD_SIZE_2K;
            case MifareClassic.SIZE_4K:
                return CardConstants.CARD_SIZE_4K;
            case MifareClassic.SIZE_MINI:
                return CardConstants.CARD_SIZE_MINI;
            default:
                return CardConstants.CARD_SIZE_UNKNOWN;
        }
    }

    public String convertBytesToString(byte[] data) {
        String tagId = "";
        for (int i = 0; i < data.length; i++) {
            tagId += String.format("%02X", data[i] & 0xff);
        }
        return tagId;
    }

    public boolean cardValidate(int t, int s) {
        if (t == MifareClassic.TYPE_CLASSIC && s == MifareClassic.SIZE_1K) {
            return true;
        }
        return false;
    }

    public CardInformation cardReadStorage(MifareClassic mfc) {
        CardInformation information = null;
        try {

            String [] pointCompany = readBlock(mfc, 3).split(CardConstants.SEPARATOR);
            String [] cardProp = readBlock(mfc, 4).split(CardConstants.SEPARATOR);
            String [] balance = readBlock(mfc, 5).split(CardConstants.SEPARATOR);

            information = new CardInformation(
                    readBlock(mfc, 1),
                    readBlock(mfc, 2),
                    pointCompany[0],
                    pointCompany[1],
                    cardProp[0],
                    cardProp[1],
                    cardProp[2],
                    balance[1],
                    readBlock(mfc, 13),
                    readBlock(mfc, 12)
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
        return information;
    }

    public void writeCard(MifareClassic mfc, Card card) {
        try {

            writeBlock(mfc, 1, card.getParam_1());
            writeBlock(mfc, 2, card.getParam_2());
            writeBlock(mfc, 3, card.getParam_3());
            writeBlock(mfc, 4, card.getParam_4());
            writeBlock(mfc, 5, card.getParam_5());
            writeBlock(mfc, 6, card.getParam_6());
            writeBlock(mfc, 7, card.getParam_7());
            writeBlock(mfc, 8, card.getParam_8());
            writeBlock(mfc, 9, card.getParam_9());
            writeBlock(mfc, 10, card.getParam_10());
            writeBlock(mfc, 11, card.getParam_11());
            writeBlock(mfc, 12, card.getParam_12());
            writeBlock(mfc, 13, card.getParam_13());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized void clearBlock(MifareClassic mfc) {
        try {
            byte[] payload = new byte[16];
            Arrays.fill(payload, (byte) 0);
            for (int sectorIndex = 0; sectorIndex < mfc.getSectorCount(); sectorIndex++) {
                boolean authSector = mfc.authenticateSectorWithKeyA(sectorIndex, MifareClassic.KEY_DEFAULT);
                if (authSector) {
                    int blockIndex = mfc.sectorToBlock(sectorIndex);
                    mfc.writeBlock(blockIndex, payload);
                    Log.d(TAG, "CLEAR -> sectorIndex: " + sectorIndex + " blockIndex: " + blockIndex);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public String readBlock(MifareClassic mfc, int sectorIndex) throws IOException {
        String block = "";
        boolean authSector = mfc.authenticateSectorWithKeyA(sectorIndex, MifareClassic.KEY_DEFAULT);
        if (authSector) {
            int blockIndex = mfc.sectorToBlock(sectorIndex);
            byte[] data = mfc.readBlock(blockIndex);
            block = convertBytes2String(data);
            //return convertBytesToString(data);
        }
        return block;
    }

    public void writeBlock(MifareClassic mfc, int sectorIndex, String content) throws IOException {
        boolean authSector = mfc.authenticateSectorWithKeyA(sectorIndex, MifareClassic.KEY_DEFAULT);
        if (authSector) {
            byte[] data = convertString2Bytes(content);
            int blockIndex = mfc.sectorToBlock(sectorIndex);
            mfc.writeBlock(blockIndex, data);
            Log.d(TAG, "Write blockIndex: " + blockIndex + " content: " + content);
        }
    }

    private String convertBytes2String(byte[] data) throws UnsupportedEncodingException {
        String ret;
        int pos = data.length;
        for (int i = data.length - 1; i >= 0; i--) {
            if (data[i] != 0) {
                break;
            }
            pos = i;
        }
        ret = new String(data, 0, pos, CHARSET);
        return ret;
    }

    private byte[] convertString2Bytes(String content) throws UnsupportedEncodingException {
        byte[] ret = new byte[16];
        byte[] buf = content.getBytes(CHARSET);
        int retLen = ret.length;
        int bufLen = buf.length;
        boolean b = retLen > bufLen;
        for (int i = 0; i < retLen; i++) {
            if (b && i >= bufLen) {
                ret[i] = 0;
                continue;
            }
            ret[i] = buf[i];
        }
        return ret;
    }

}
